use super::models;
use crate::repositories;
use beteran_common_rust as bcr;
use beteran_protobuf_rust as bpr;
use prost::Message;
use std::net::{IpAddr, Ipv4Addr};

pub struct Composition {
  connection_broker: nats::asynk::Connection,
  member_repository: repositories::member::repository::Repository,
}

impl std::fmt::Debug for Composition {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Composition of members").finish()
  }
}

impl Composition {
  ///
  pub fn new(connection_broker: nats::asynk::Connection) -> Composition {
    Composition {
      connection_broker,
      member_repository: repositories::member::repository::Repository::new(),
    }
  }

  ///
  pub async fn select_by_member_id(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
  ) -> Result<Option<models::Member>, bcr::error::rpc::Error> {
    let m = match self
      .member_repository
      .select_by_member_id(conn, member_id)
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })? {
      Some(m) => m,
      None => {
        return Ok(None);
      }
    };

    let ss_get_member_req = bpr::ss::member::GetMemberBRequest {
      client: Some(bpr::models::core::network::Client {
        client_ip: self
          .connection_broker
          .client_ip()
          .unwrap_or(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)))
          .to_string(),
        site_url: None,
        access_token: None,
      }),
      request: Some(bpr::ss::member::get_member_b_request::Request {
        id: m.member_id.to_string(),
      }),
    };

    let ss_get_member_res_msg = self
      .connection_broker
      .request(
        bpr::ss::member::SUBJECT_GET_MEMBER_B,
        ss_get_member_req.encode_to_vec(),
      )
      .await
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })?;

    let ss_get_member_res = bpr::ss::member::GetMemberBResponse::decode(
      ss_get_member_res_msg.data.as_slice(),
    )
    .map_err(|e| {
      bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
        code: bpr::protobuf::rpc::Error::SERVER_00,
        message: format!("server {}", e),
        data: None,
      })
    })?;

    if let Some(e) = ss_get_member_res.error {
      return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
        code: bpr::protobuf::rpc::Error::SERVER_00,
        message: format!("server {}", e),
        data: None,
      }));
    }

    let mm = match ss_get_member_res.result {
      Some(r) => match r.member {
        Some(_m) => Some(models::Member {
          id: m.id,
          balance: m.balance,
          balance_bota: m.balance_bota,
          balance_sum: m.balance_sum,
          companies: m.companies,
          oriental_play: m.oriental_play,
          member: _m,
          created_at: m.created_at,
          updated_at: m.updated_at,
        }),
        None => None,
      },
      None => None,
    };

    Ok(mm)
  }

  ///
  pub async fn select_b_by_member_id(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
  ) -> Result<Option<models::MemberModel>, bcr::error::rpc::Error> {
    let m = match self
      .member_repository
      .select_by_member_id(conn, member_id)
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })? {
      Some(m) => m,
      None => {
        return Ok(None);
      }
    };

    let ss_get_member_req = bpr::ss::member::GetMemberRequest {
      client: Some(bpr::models::core::network::Client {
        client_ip: self
          .connection_broker
          .client_ip()
          .unwrap_or(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)))
          .to_string(),
        site_url: None,
        access_token: None,
      }),
      request: Some(bpr::ss::member::get_member_request::Request {
        id: m.member_id.to_string(),
      }),
    };

    let ss_get_member_res_msg = self
      .connection_broker
      .request(
        bpr::ss::member::SUBJECT_GET_MEMBER,
        ss_get_member_req.encode_to_vec(),
      )
      .await
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })?;

    let ss_get_member_res = bpr::ss::member::GetMemberResponse::decode(
      ss_get_member_res_msg.data.as_slice(),
    )
    .map_err(|e| {
      bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
        code: bpr::protobuf::rpc::Error::SERVER_00,
        message: format!("server {}", e),
        data: None,
      })
    })?;

    if let Some(e) = ss_get_member_res.error {
      return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
        code: bpr::protobuf::rpc::Error::SERVER_00,
        message: format!("server {}", e),
        data: None,
      }));
    }

    let mm = match ss_get_member_res.result {
      Some(r) => match r.member {
        Some(_m) => Some(models::MemberModel {
          id: m.id,
          balance: m.balance,
          balance_bota: m.balance_bota,
          balance_sum: m.balance_sum,
          companies: m.companies,
          oriental_play: m.oriental_play,
          member: _m,
          created_at: m.created_at,
          updated_at: m.updated_at,
        }),
        None => None,
      },
      None => None,
    };

    Ok(mm)
  }
}
