//!
//!
use super::{models, schema::api_kgon_synchronization_history};
use beteran_common_rust as bcr;
use diesel::result::Error;
use diesel::{prelude::*, sql_query};

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of api_kgon_synchronization_history")
      .finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_member: &models::NewSynchronizationHistory,
  ) -> Result<models::SynchronizationHistory, Error> {
    let inserted = diesel::insert_into(api_kgon_synchronization_history::table)
      .values(new_member)
      .get_result::<models::SynchronizationHistory>(conn)?;

    Ok(inserted)
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: i64,
  ) -> Result<Option<models::SynchronizationHistory>, Error> {
    match api_kgon_synchronization_history::table
      .find(id)
      .first::<models::SynchronizationHistory>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_latest_by_item(
    &self,
    conn: &diesel::PgConnection,
    item: String,
  ) -> Result<Option<models::SynchronizationHistory>, Error> {
    let query = "
SELECT
  id,
  item,
  start_at,
  complete_at,
  code,
  message,
  data,
  created_at
FROM 
  (
  SELECT
    id,
    item,
    start_at,
    complete_at,
    code,
    message,
    data,
    created_at,
    row_number() OVER(PARTITION BY item ORDER BY created_at DESC) AS rn
  FROM
    api_kgon_synchronization_history
  ) t
WHERE t.rn = 1 AND item = $1
    ";

    match sql_query(query)
      .bind::<diesel::sql_types::Text, _>(item)
      .get_result::<models::SynchronizationHistory>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = api_kgon_synchronization_history::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = &s.item {
        q = q.filter(api_kgon_synchronization_history::dsl::item.eq(sp));
      }
      if let Some(sp) = s.code {
        q = q.filter(api_kgon_synchronization_history::dsl::code.eq(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::SynchronizationHistory>, Error> {
    let mut q = api_kgon_synchronization_history::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = &s.item {
        q = q.filter(api_kgon_synchronization_history::dsl::item.eq(sp));
      }
      if let Some(sp) = s.code {
        q = q.filter(api_kgon_synchronization_history::dsl::code.eq(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::pagination::Sort::ASC(property) => match property.as_str() {
            "item" => {
              q = q.order_by(api_kgon_synchronization_history::item.asc());
            }
            "start_at" => {
              q = q.order_by(api_kgon_synchronization_history::start_at.asc());
            }
            "complete_at" => {
              q = q.order_by(api_kgon_synchronization_history::complete_at.asc());
            }
            "code" => {
              q = q.order_by(api_kgon_synchronization_history::code.asc());
            }
            "created_at" => {
              q = q.order_by(api_kgon_synchronization_history::created_at.asc());
            }
            _ => {}
          },
          bcr::pagination::Sort::DESC(property) => match property.as_str() {
            "item" => {
              q = q.order_by(api_kgon_synchronization_history::item.desc());
            }
            "start_at" => {
              q = q.order_by(api_kgon_synchronization_history::start_at.desc());
            }
            "complete_at" => {
              q = q.order_by(api_kgon_synchronization_history::complete_at.desc());
            }
            "code" => {
              q = q.order_by(api_kgon_synchronization_history::code.desc());
            }
            "created_at" => {
              q = q.order_by(api_kgon_synchronization_history::created_at.desc());
            }
            _ => {}
          },
        };
      }
    }

    q.load::<models::SynchronizationHistory>(conn)
  }

  ///
  pub fn delete(&self, conn: &diesel::PgConnection, id: i64) -> Result<u64, Error> {
    use api_kgon_synchronization_history::dsl;

    diesel::delete(api_kgon_synchronization_history::table.filter(dsl::id.eq(id)))
      .execute(conn)
      .map(|c| c as u64)
  }
}
